/* -*- c-file-style: "k&r"; c-basic-offset: 4; indent-tabs-mode: nil -*-
 * 
 * distcc -- A simple distributed compiler system
 *
 * Copyright (C) 2004 by Martin Pool <mbp@samba.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

                /* Time spent talking to a wall is time better
                 * spent demolishing it.     -- Ilan
                 */


/* timeouts on connections, etc */

#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <setjmp.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#include "distcc.h"
#include "exitcode.h"
#include "trace.h"
#include "state.h"


static void dcc_timeout_handler(int number);
static jmp_buf timeout_jmpbuf;


static void
dcc_timeout_handler(int signum)
{
     (void) signum;
     longjmp(timeout_jmpbuf, EXIT_TIMEOUT);
}


/* Limit the time the entire connect/resolve operation can take.
 *
 * If the timeout expires, this function will return a second time
 * through a jmpbuf with an error.
 *
 * This function may be called again to reset the timer. */
int
dcc_timeout_arm(const int timeout, int phase)
{
     static enum dcc_phase saved_phase;

     saved_phase = phase;

     if (setjmp(timeout_jmpbuf)) {
          /* setjmp return through here if it timed out. */
          rs_log_error("%s timeout", dcc_get_phase_name(saved_phase));
          return EXIT_TIMEOUT;
     }

     if (signal(SIGALRM, dcc_timeout_handler) == SIG_ERR) {
          rs_log_error("error registering signal: %s", strerror(errno));
          return EXIT_DISTCC_FAILED;
     }

     rs_trace("turned on timeout for %ds", timeout);
     alarm(timeout);

     return 0;
}


int
dcc_timeout_disarm(void)
{
     alarm(0);
     return 0;
}


